// ===========================================================================
// ==           Rock class : Implementation                                 ==
// ==              Atchoum et X. Pinguin 16 Avril 2000                      ==
// ===========================================================================

#include "Actors/Rock.h"
#include "Data/ActionId.h"
#include "Render/RenderRock.h"
#include "Data/DataRoom.h"

static DataRoom trashRoom(0,0,0);

Rock::Rock(DataRoom& room, int x, int y) : 
  Monster(x,y),
  _dx(0), _dy(0), _nact(0),
  _ia(room, _data)
{
  room.getDynData().push(_data,x,y);
  _renderObj = new RenderRock;
  _renderObj->setPosition(x<<5, y<<5);
  _canBeControled = true;
  _hasMoved = false;
  _isMortal = false;
}
// ===========================================================================
Rock::Rock(int x, int y) :
  Monster(x,y),
  _dx(0), _dy(0), _nact(0),
  _ia(trashRoom, _data)
{
}  
// ===========================================================================
Rock::~Rock()
{
  delete _renderObj;
}
// ===========================================================================
bool
Rock::mustRedraw() const
{
  if (_data.getState()!=DONOTHING)
    return true;
  return false;
}
// ===========================================================================
void
Rock::doAction()
{
  _isMortal = false;
  switch(_data.getState())
  {
  case MOVEDOWN:
    //    cerr << "Roche tombe\n";
    _isMortal = true;
    doFall();
    break;
  case MOVELEFT:
    doPush(-1);
    break;
  case MOVERIGHT:
    doPush(+1);
    break;
  default:
    break;
  }
  if (_data.getState()!=DONOTHING)
	  _renderObj->setPosition((_data.getXPos()<<5)+_dx, (_data.getYPos()<<5)+_dy);
}
// ===========================================================================
bool
Rock::setAction(int action)
{
  bool flag = false;
  flag = true;
  flag &= Actor::setAction(action);
  if (action != DONOTHING)
    _canBeControled = false;
  else
    _canBeControled = true;
  return flag;
}
// ===========================================================================
bool
Rock::setAction()
{
  if (canControling())
  {  
    _hasMoved = false;
    int action = _ia.chooseAction();
     return setAction(action);
  }
  return true;
}
// ===========================================================================
void
Rock::doFall()
{
  _nact++;
  _dy += 2;
  if (_nact&16)
  {
    _data.setState(DONOTHING);
    _nact = 0;
    _canBeControled = true;
    _data.move(0,1);
    _dy = 0;
    _hasMoved = true;
  }
  else
    _hasMoved = false;
}
// ===========================================================================
void
Rock::doPush(int dx)
{
  _nact++;
  _dx += 4*dx;
  if (_nact&8)
  {
    _data.setState(DONOTHING);
    _nact = 0;
    _canBeControled = true;
    _data.move(dx,0);
    _dx = 0;
    _hasMoved = true;
  }
  else
    _hasMoved = false;
}  
// ===========================================================================
Actor* 
Rock::clone(DataRoom& room, int x, int y, int ) const
{
  return new Rock(room, x, y);
}
