#include <iostream>
#include <strstream>
#include <stdio.h>
#include <stdlib.h>
#include <fstream>

#include <SDL.h>
#include <SDL_timer.h>

#include "Scenary/Scenary.h"
#ifdef MIKMOD
# include "Music/ModPlayer.h"
#endif
#include "Graphism/Screen.h"
#include "Graphism/Text.h"

#include "Actors/Rock.h"
#include "Actors/Player.h"

#include "Object/Nothing.h"
#include "Object/Wall.h"
#include "Object/Sand.h"
#include "Object/Diamond.h"

char mapLvl[12][21];
int maxLevel;

struct strHighScore
{
	std::string name;
  int    lvl;
  int    score;
  strHighScore() : name("None"),lvl(0),score(0)
  {}
};

strHighScore highScore[5];

#ifdef MIKMOD
ModPlayer music;
#endif

int showMainScreen(Screen& ecr)
{
  int choix = 0;
  ecr.clear();
  Image fondImg ("Graphisme/Intro.bmp");
  ecr << fondImg;
  ecr.blit();
  while (choix==0)
  {
    SDL_Event event;
    while(SDL_PollEvent(&event))
    {
      switch(event.type)
      {
      case SDL_KEYDOWN:
      {
	switch(event.key.keysym.sym)
	{
	case SDLK_KP1:
	case SDLK_1:
	  choix = 1;
	  break;
	case  SDLK_KP2:
	case  SDLK_2:
	  choix = 2;
	  break;
	case  SDLK_KP3:
	case  SDLK_3:
	  choix = 3;
	  break;
	case  SDLK_KP4:
	  choix = 4;
	  break;
	default:
	  break;
	}
      }
      break;
      default:
	break;
      }
    }
#ifdef MIKMOD
     if (!music.isPlaying())
    {
      music.load("Music/storm.mod");
      music.play();
      }
#endif
  }
#ifdef MIKMOD
  music.stop();
#endif
  return choix;
}

bool
loadLevel(std::ifstream& file, int score, Scenary& scene)
{
  Nothing none;
  Wall    wall;
  Sand    sand;
  Diamond diam;
  int i;
  for (i = 0; i<12; i++)
  {
    file.getline(mapLvl[i],22);
  }

  for (i = 0; i<12; i++)
    for (int j = 0; j<20; j++)
    {
      switch(mapLvl[i][j])
      {
      case ' ':
	scene.addObject(none, j,i);
	break;
      case 'm':
	scene.addObject(wall, j,i);
	break;
      case 'h':
	scene.addObject(sand, j,i);
	break;
      case 'r':
	scene.addActor(Rock(j,i), j, i);
	break;
      case 'p':
	scene.addObject(diam, j,i);
	break;
      }
    }

  int ndiam, xp, xy, energy;
  file >> ndiam >> energy >> xp >> xy;
  scene.addPlayer(score, energy,xp,xy);
  return true;
}

void updateHighScore(int score, int lvl)
{
  int i;
  for (i = 0; i< 5; i++)
    if (score>highScore[i].score)
      break;
  if (i!=5)
  {
    for (int j = 3; j>=i; j--)
      highScore[j+1] = highScore[j];
    highScore[i].score = score;
    highScore[i].lvl   = lvl;
#ifdef POSIX
    highScore[i].name = getenv("USER");
#else
    highScore[i].name =	"toto";
#endif
  }
}

void showHighScores(Screen& ecr)
{
  char temp[32];
  ecr.clear();
  Image fondImg ("Graphisme/highscore.bmp");
  ecr << fondImg;
  for (int i = 0; i < 5; i++)
  {
    sprintf(temp,"%d.",i+1);
    std::string strRank(temp);
    Text tRank(strRank,100, 200+i*48);
    Text tName(highScore[i].name, 150, 200+i*48);
    sprintf(temp,"%d",highScore[i].score);
    std::string strScore(temp);
    Text tScore(strScore, 360, 200+i*48);
    sprintf(temp, "%d",highScore[i].lvl+1);
    std::string strLevel(temp);
    Text tLevel(strLevel, 548, 200+i*48);
    ecr << tRank << tName << tScore << tLevel;
  }
  ecr.blit();
  bool flag = false;
  while ( !flag)
  {
    SDL_Event event;
    while(SDL_PollEvent(&event))
    {
      switch(event.type)
      {
      case SDL_KEYDOWN:
      {
	flag = true;
      }
      break;
      default:
	break;
      }
    }
#ifdef MIKMOD
   if (!music.isPlaying())
    {
      music.load("Music/theend.mod");
      music.play();
    }
#endif
  }
#ifdef MIKMOD
  music.stop();
#endif
}  

#define TICK_INTERVAL 60
Uint32 TimeLeft(void)
{
	static Uint32 next_time=0;
	Uint32 now;
	now=SDL_GetTicks();
	if (next_time <= now)
	{
		next_time=now+TICK_INTERVAL;
		return (0);
	}
	return (next_time-now);
}

void play(const std::string& game, Screen& monEcran)
{
  std::ifstream file(game.c_str());
  char trash[16];
  // Nbre maximal de niveau.
  file >> maxLevel;  
  file.getline(trash,16);
  // Chargement graphisme.
  Image fondImg ("Graphisme/Fond.bmp");
  Image fondImg2 ("Graphisme/Fond2.bmp");
  Sprite fondSpr( fondImg, 1, 1);
  Sprite fondSprEner( fondImg2, 1, 1);
  Sprite fondSprPom( fondImg2, 1, 1);
  Sprite fondSprScore( fondImg2, 1, 1);
  fondSpr.setNewPos(0,384);
  fondSprEner.setNewPos(216,436);
  fondSprPom.setNewPos(448,436);
  fondSprScore.setNewPos(16,436);

  Text textScore("Score", 16, 396);
  Text textEner ("Energy",216, 396);
  Text textPaleo("Paleo", 448, 396);

//  SDL_Event event;
  monEcran.blit();
  // Chargement de la pice.
  int level = 0;
  int score = 0;
  int energy;

  do
  {
    bool OK = true;
    Scenary scene(20,12);
    // Chargement d'un niveau particuliers.
    loadLevel(file, score, scene);
    // Et de sa musique associe.
    char musicName[128];
    char musicBaseName[64];
    file.getline(musicBaseName,63);
    file.getline(musicBaseName,63);
    sprintf(musicName,"Music/%s",musicBaseName);
    monEcran.clear();
    // ==============================
    monEcran << fondSpr << textScore << textEner << textPaleo;
    
    char temp[32];
    while(OK)
    {
      OK = !(scene.play(monEcran));
      OK &= (scene.getPlayer().getEnergy()!=0);

      score = scene.getScore();
      sprintf(temp,"%d",score);
      std::string strScore(temp);
      Text tScore(strScore,16, 436);
      monEcran << fondSprScore << tScore;

      energy = scene.getPlayer().getEnergy();
      sprintf(temp,"%d",energy);
      std::string strEner(temp);
      Text tEner(strEner,216, 436);
      monEcran << fondSprEner << tEner;

      int pommes = scene.getNbDiam();
      sprintf(temp,"%d",pommes);
      std::string strPom(temp);
      Text tPom(strPom,448, 436);
      monEcran << fondSprPom << tPom;

      monEcran.blit();

      SDL_Delay(TimeLeft());
//	        SDL_Delay(0);

#ifdef MIKMOD      
	if (!music.isPlaying())
      {
	music.load(musicName);

	music.play();
      }
#endif	
    }
#ifdef MIKMOD
    music.stop();
#endif
    if (energy!=0)
    {
      score += 100*energy;
      level++;
    }
  } while ((energy!=0)&&(level<maxLevel));
  if (level==maxLevel)
  {
    std::cout << "Felicitations\n";    
  }
  else
  {
    std::cout << "Perdu !\n";
  }
  std::cout << "level = " << level << " et score = " 
       << score << std::endl;
  updateHighScore(score, level);
}

int main(int argc, char** argv)
{
 
  Screen monEcran(640,480,16,true);
 
  std::string DefGame = "Data/Levels.dat";
  std::string gameName = DefGame;
  if (argc==2)
    gameName = std::string(argv[1]);
  int choix = 0;
  do
  {
    choix = showMainScreen(monEcran);
    switch(choix)
    {
    case 1:
      play(gameName,monEcran);
      break;
    case 2:
      break;
    case 3:
      showHighScores(monEcran);
      break;
    default:
      break;
    }
   } while ( choix != 4);
  return 0;
}


