// ===========================================================================
// ==            DynamicRoom : Definition                                   ==
// ==             Atchoum & X. Pinguin 22 April 2000                        ==
// ===========================================================================
#ifndef _DYNAMICROOM_H
#define _DYNAMICROOM_H

#include <list>

class DataActor;

/** 
*/
class DynamicRoom
{
public:
  /** Definition of a container class. */
	typedef std::list<DataActor*> Container;
  /** Definition of a read/write iterator. */
  typedef Container::iterator iterator;
  /** Definition of a read only iterator. */
  typedef Container::const_iterator const_iterator;
  /** Constructor. The last argument is used only in 3D games. */
  DynamicRoom(int lx, int ly, int lz=0);
  /** 
      Destructor. Destroy nothing. The DataActors in container is only
      a weak link.
  */
  ~DynamicRoom();
  /**
     Create a r/w iterator on the beginning of the container.
  */
  iterator begin();
  /**
     Create a read only iterator on the beginning of the container.
  */
  const_iterator begin() const;
  /**
     Create a r/w iterator on the end of the container.
  */
  iterator end();
  /**
     Create a read only iterator on the end of the container.
  */
  const_iterator end() const;
  /**
   */
  DataActor* getDataActor(int x, int y, int z = 0);
  /**
   */
  const DataActor* getDataActor(int x, int y, int z = 0) const;
  /**
     Add a new actor in the room at a position (x,y,z)
   */
  void push(DataActor& actor, int x, int y, int z = 0);
  /**
     Remove a actor from the room ( killed, etc... )
  */
  bool remove(DataActor& actor);
private:
  //
  Container _datas;
  //
  int _lx, _ly, _lz;
};
// ==================== INLINE SECTION =======================================
inline
DynamicRoom::DynamicRoom(int lx, int ly, int lz) :
  _datas(),
  _lx(lx), _ly(ly), _lz(lz)
{
}
// ===========================================================================
inline DynamicRoom::iterator
DynamicRoom::begin()
{
  return _datas.begin();
}
// ===========================================================================
inline DynamicRoom::const_iterator
DynamicRoom::begin() const
{
  return _datas.begin();
}
// ===========================================================================
inline DynamicRoom::iterator
DynamicRoom::end()
{
  return _datas.end();
}
// ===========================================================================
inline DynamicRoom::const_iterator
DynamicRoom::end() const
{
  return _datas.end();
}
//
// ===========================================================================
#endif
