// =============================================================================
// ==                     Class StaticRoom : Definition                       ==
// ==                   Atchoum & X. Pinguin  3 Mars 2000                     ==
// =============================================================================
#ifndef _STATICROOM_H
#define _STATICROOM_H
#include <vector>

class DataObject;

/** Cette classe reprsente les objets statiques d'une pice, c'est  dire
    les objets comme les murs qui sont des objets immuables de la pice.
*/
class StaticRoom
{
 public:
  /** Dfinition du container de la classe. */
	 typedef std::vector<DataObject*> Container;
  /** On dfinit un itrateur dans la classe DataRoom qui permet de parcourir
      tout les objets donnes des lments statiques de la pice. */
  typedef Container::iterator iterator;
  /** Mme chose mais pour un objet constant. */
  typedef Container::const_iterator const_iterator;
  /** Ce constructeur construit une pice vide d'objets statiques mais qu'on
      pourra remplir d'lments immuables par la suite. Les paramtres donns
       la pice sont ses dimensions (lx,ly,lz). Par dfaut, pour les jeux en
      deux dimensions, lz est mis  0.
  */
  StaticRoom( int lx, int ly, int lz = 0 );
  /** The destructor doesn't destroy the data objects. */
  ~StaticRoom();
  /** 
      On place un itrateur en dbut de liste pour lire et crire..
  */
  iterator begin();
  /** 
      On place un itrateur en dbut de liste pour lire uniquement. 
  */
  const_iterator begin() const;
  /** On place un itrateur en fin de liste pour lire/crire. */
  iterator end();
  /** On place un itrateur en fin de liste pour lire uniquement. */
  const_iterator end() const;
  /** Accs direct  un lment du dcor en criture/lecture. */
  DataObject* getObj(int x, int y, int z=0);
  /** Accs direct  un lment du dcor en lecture seulement. */
  const DataObject* getObj(int x, int y, int z=0) const;
  /** Rajouter un lment statique  la scne. */
  void insert(DataObject& obj, int x, int y, int z=0);
  /** Dtruire un lment statique de la scne. Renvoie faux si il
   n'y a pas d'lments  l'endroit indiqu.*/
  bool remove(int x, int y, int z=0);
  /** Dtruire un lment. */
  bool remove(const DataObject& obj);
  /** Get the dimension of the room. */
  void getDim(int& lx, int& ly) const;
  /** Get the dimension of the room. */
  void getDim(int& lx, int& ly, int& lz) const;
 private:
  int _lx, _ly, _lz;
  Container _staticObjs;
};
// ======================== INLINE SECTION ================================
inline const DataObject*
StaticRoom::getObj(int x, int y, int z) const
{
  return _staticObjs[x+y*_lx+z*_lx*_ly];
}
// ========================================================================
inline DataObject*
StaticRoom::getObj(int x, int y, int z)
{
  return _staticObjs[x+y*_lx+z*_lx*_ly];
}
// ========================================================================
inline StaticRoom::iterator
StaticRoom::begin()
{
  return _staticObjs.begin();
}
// ========================================================================
inline StaticRoom::const_iterator
StaticRoom::begin() const
{
  return _staticObjs.begin();
}
// ========================================================================
inline StaticRoom::iterator
StaticRoom::end()
{
  return _staticObjs.end();
}
// ========================================================================
inline StaticRoom::const_iterator
StaticRoom::end() const
{
  return _staticObjs.end();
}
// ========================================================================
inline void 
StaticRoom::getDim(int& lx, int& ly) const
{
  lx = _lx; ly = _ly;
}
// ========================================================================
inline void 
StaticRoom::getDim(int& lx, int& ly, int& lz) const
{
  lx = _lx; ly = _ly; lz = _lz;
}
#endif
