// ========================================================================
// ==             Implmentation de la classe Image                      ==
// ==              Atchoum & X Pinguin  6 Fevrier 2000                   ==
// ========================================================================

#include <stdlib.h>
#include <SDL.h>

#include "Graphism/Image.h"
#include "Graphism/Screen.h"
#include "Util/ErrLog.h"

Image::Image(const std::string& filename) : _image(NULL),
					    _cptRef(new int(1))
{
  _image= SDL_LoadBMP(filename.c_str());
   if (_image==NULL)
   {
	   eLog << "Probleme de lecture du fichier" << std::endl;
	   eLog << SDL_GetError() << std::endl;
     exit( EXIT_FAILURE );
   }
}
// ========================================================================
Image::Image(const Image& img) : _image(img._image),
				 _cptRef(img._cptRef)
{
  (*_cptRef)++;
}
// ========================================================================
Image::~Image()
{
  (*_cptRef)--;
  if ((*_cptRef)==0)
  {
    SDL_FreeSurface(_image);
    delete _cptRef;
  }
}
// ========================================================================
const Image&
Image::operator= (const Image& img)
{
  if ( &img != this )
  {
    (*_cptRef)--;
    if ((*_cptRef)==0)
    {
      SDL_FreeSurface(_image);
      delete _cptRef;
    }
    _image  = img._image;
    _cptRef = img._cptRef;
    (*_cptRef)++;
  }
  return (*this);
}
// ========================================================================
Uint32 
Image::readPixel(int x, int y) const
{
 return (*(((Uint8*)_image->pixels)+
		     y*_image->pitch+x*((_image)->format)->BytesPerPixel));
}
// ========================================================================
void 
Image::setTransparentPixel(int fromx, int fromy)
{
  if ( SDL_SetColorKey( _image,
		        (SDL_SRCCOLORKEY|SDL_RLEACCEL), 
		        readPixel(fromx,fromy)          ) !=0 )
  {
	  eLog << "probleme d'assignation de la transparence" << std::endl;
  }
#ifdef DEBUGTRANSPARENCE
  cout << readPixel(fromx,fromy) << endl;
#endif
}
// ========================================================================
void 
Image::setTransparentPixel(Uint8 pelCode)
{
  if ( SDL_SetColorKey  ( _image,
			  (SDL_SRCCOLORKEY|SDL_RLEACCEL),
			  pelCode)                         !=0) 
  {
	  eLog << "probleme d'assignation de la transparence" << std::endl;
  }
}
// ===========================================================================
Screen&
operator<<(Screen& scr, const Image& img)
{
  SDL_Surface& frameSet = *(img._image);
  SDL_Rect srect;
  srect.x = srect.y = 0;
  srect.w = frameSet.w;
  srect.h = frameSet.h;
  
  SDL_BlitSurface(&frameSet,
		  &srect, scr.getScreen(),
		  &srect );
  return scr;
}
// ===========================================================================
