// ========================================================================
// ==             Class Sprite : Definition                              ==
// ==              Atchoum & X Pinguin  6 Fevrier 2000                   ==
// ========================================================================
#ifndef _SPRITE_H_
#define _SPRITE_H_

#include "Graphism/Image.h"

class Screen;

/**
   La classe Sprite permet d'afficher des images "animes"  partir d'un
   simple fichier bitmap ! Le fichier bitmap devra se prsenter comme suit :

   |img1|img2|...|imgn|

   c'est  dire comme des images de l'animation mis bout  bout ( de gauche
    droite ). Cete image devra pralablement charge avec la classe Image.
*/
class Sprite
{
 public:
  /** Constructeur. 

      @param img       L'image bitmap reprsentant l'animation du sprite.
      @param nbrFrame  Le nombre de squence pour l'animation du sprite.
      @param initFrame La position dans la squence de ce sprite.
      @param x         Coordonne abcisse  initiale du sprite.
      @param y         Coordonne ordonne initiale du sprite.
   */
  
  Sprite(Image& img,int nbrFrame, int nbPosition, int initFrame = 0, 
	 bool isPingPong = false,
	 int x=0, int y=0);
  /** Pour passer  l'image suivante de l'animation. */
  void nextFrame();
  /** set Frame */
  void setFrame(int initFrame);
  /** set Position ( 0 to nbPos-1 )*/
  void setPosition(int position);
  /** Obtenir le rectangle destinataire ?*/
  SDL_Rect * getDst() const;
  /** Obtenir le rectangle source ? */
  SDL_Rect * getSrc() const;
  /** Obtenir l'image globale */
  const Image&  getImage() const;
  /** Obtenir la largeur d'une image du sprite. */
  int getWidth() const;
  /** Obtenir la hauteur d'une image du sprite. */
  int getHeight() const;
  /** Obtenir l'image globale */
  Image&  getImage();
  /** Repositionner le sprite sur l'cran en (x,y). */
  void setNewPos(int x, int y);
 private:
  mutable SDL_Rect _destRec;
  mutable SDL_Rect _srcRec;
  Image _image;
  int _nbrFrame;
  int _currentFrame;
  int _nbPosition;
  int _currentPosition;
  int _incFrame;
  bool _isPingPong;
  friend Screen& operator <<(Screen& scr, const Sprite& spr);
};

// ================= INLINE SECTION =======================================
inline SDL_Rect * Sprite::getDst() const
{
  return (& _destRec);
}
// ========================================================================
inline SDL_Rect * Sprite::getSrc() const
{
  return (& _srcRec);
}
// ========================================================================
inline Image&
Sprite::getImage()
{
  return _image;
}// ===========================================================================
inline const Image&
Sprite::getImage() const
{
  return _image;
}
// ===========================================================================
inline int 
Sprite::getWidth() const
{
  return _destRec.w;
}
// ===========================================================================
inline int
Sprite::getHeight() const
{
  return _destRec.h;
}
#endif
