
// ===========================================================================
// ==            Scenary implementation                                     ==
// ==             Atchoum & X. Pinguin 22 April 2000                        ==
// ===========================================================================

#include "Scenary.h"

#include "Data/DataRoom.h"
#include "Data/DynamicRoom.h"
#include "Data/IdObject.h"
#include "Actors/Actor.h"
#include "Actors/Player.h"
#include "Actors/Monster.h"
#include "Render/RenderRoom.h"
#include "Render/RenderObject.h"
#include "Render/RenderActor.h"
#include "Data/DataActor.h"
#include "Data/ActionId.h"
#include "Object/Object.h"
#include "Object/Nothing.h"


Scenary::Scenary(int lx, int ly, int lz ) :
  _data(lx,ly,lz), _render(lx,ly,lz),
  _lx(lx), _ly(ly), _lz(lz),
  _nbOfDiamonds(0),
  _player(NULL)
{
}
// ===========================================================================
Scenary::~Scenary()
{
  for (std::list<Actor*>::iterator iteractor = _listOfActor.begin();
       iteractor != _listOfActor.end(); iteractor ++)
    delete *iteractor;
  for (std::list<Object*>::iterator iterobject = _vectOfObject.begin();
       iterobject != _vectOfObject.end(); iterobject ++)
    delete *iterobject;
}
// ===========================================================================
bool
Scenary::addActor(const Actor& actor, int x, int y)
{
  Nothing none;
  Actor* actorClone = actor.clone(_data, x, y);
  _listOfActor.push_back(actorClone);
  _render.addActor(actorClone->getRenderObj(),x,y);
  addObject(none,x,y);
	return true;
}
// ===========================================================================
bool
Scenary::addPlayer(int score, int energy, int x, int y)
{
  _score = score;
  _player = new Player(_data, x,y, energy);
  _listOfActor.push_back((Actor* const)_player);
  _render.addActor(_player->getRenderObj(),x,y);
  return true;
}
// ===========================================================================
bool
Scenary::addObject(const Object& objr, int x, int y)
{
  Object* obj = objr.clone(x,y);
  _vectOfObject.push_back((Object *const)obj);
  _render.addObject(obj->getRender(),x,y);
  _data.getStaData().insert(obj->getData(),x,y);
  if (obj->getData().getId()==DIAM)
    _nbOfDiamonds++;
  return true;
}
// ===========================================================================
bool
Scenary::rmObject(int x, int y, int z)
{
  const DataObject* dObj = _data.getStaData().getObj(x,y,z);
  if (dObj != NULL)
  {
    std::list<Object*>::iterator itrObj;
    for (std::list<Object*>::iterator iter = _vectOfObject.begin();
	 iter != _vectOfObject.end(); iter++)
    {
      if ((&(*iter)->getData()) == dObj)
      {
	itrObj = iter;
	break;
      }
    }
    assert(itrObj != _vectOfObject.end());
    _vectOfObject.erase(itrObj);
    _render.removeObject(x,y,z);
    _data.getStaData().remove(x,y,z);
  }
  return (dObj != NULL);
}
// ===========================================================================
void
Scenary::ValidAction(Actor& actor) const
{
  bool flag = true;
  DataActor& dActor = actor.getData();
  int action = dActor.getState();
  if (action != DONOTHING)
  {
    int dx,dy;
    dx = 0; dy = 0;
    switch(action)
    {
    case MOVELEFT:
      dx = -1;
      break;
    case MOVERIGHT:
      dx = +1;
      break;
    case MOVEDOWN:
      dy = +1;
      break;
    case MOVEUP:
      dy = -1;
      break;
    default:
      break;
    }
    int x,y;
    dActor.getPosition(x,y);
    const DataObject* obj = _data.getStaData().getObj(x+dx,y+dy);
    if (obj!=NULL)
    {
      // Si ce n'est pas la joueur.
      if ((dActor.getFaction() != 1)&&(obj->getId()!=NONE))
      {
	flag = false;
      }
      // Ou si c'est un obstacle.
      else if (obj->isObstacle())
      {
	flag = false;
      }
    }
    // Gestion interaction des acteurs.
    //  ---------------------------------------------------
    for (std::list<Actor*>::const_iterator itr = _listOfActor.begin();
	 itr != _listOfActor.end(); itr++)
    {
      if (*itr != &actor)
      {
	DataActor& data = (*itr)->getData();
	int ax = data.getXPos();
	int ay = data.getYPos();
	int az = data.getZPos();
	if ((x==ax-dx)&&(y==ay-dy))
	{
	  if (dActor.getFaction()==data.getFaction())
	    flag = false;
	  else if (dActor.getFaction()==1)
	  {
	    if (action==MOVELEFT)
	    {
	      if ((*itr)->getData().getState() == DONOTHING)
		(*itr)->setAction(MOVELEFT);
	      ValidAction(*(*itr));
	      if ((*itr)->getData().getState() == DONOTHING)
		flag = false;
	    }
	    else if (action==MOVERIGHT)
	    {
	      if ((*itr)->getData().getState() == DONOTHING)
		(*itr)->setAction(MOVERIGHT);
	      ValidAction(*(*itr));
	      if ((*itr)->getData().getState() == DONOTHING)
		flag = false;
	    }
	    else
	      flag = false;
	  }
	  else 
	    {
	      Monster* monster = (Monster*)(&actor);
	      if ((action==MOVEDOWN)&&(monster->isMortal()==false))
		flag = false;
	    }
	}
      }
    }
  }
  if (!flag)
    actor.setAction(DONOTHING);
}
// ===========================================================================
bool
Scenary::play(Screen& scr)
{
  // Dcision des actions pour les acteurs...
  for (std::list<Actor*>::iterator itrsetact = _listOfActor.begin();
       itrsetact != _listOfActor.end(); itrsetact++)
  {
    //    cerr << "Choix Action\n";
    (*itrsetact)->setAction();
  }
  // On vrifie la validit des actions dcides par les acteurs avec les
  // dcors statiques et dynamiques.
  for (std::list<Actor*>::iterator itrvalact = _listOfActor.begin();
       itrvalact != _listOfActor.end(); itrvalact++)
  {
    ValidAction(*(*itrvalact));
  }
    
  // On excute les actions dfinitives.
  //    1. En premier lieu, on enlve tous les acteurs de la scne.
  //    -----------------------------------------------------------
  for (std::list<Actor*>::iterator itrremact = _listOfActor.begin();
       itrremact != _listOfActor.end(); itrremact++)
  {
    DataActor& data = (*itrremact)->getData();
    int x = data.getXPos();
    int y = data.getYPos();
    int z = data.getZPos();
    (*itrremact)->doAction();
    if ((*itrremact)->hasMoved())
    {
      _render.removeActor(x,y,z);
    }
  }
  //    2. En deuxime lieu, on excute les actions des acteurs.
  //       en les rajoutant sur la scne.
  //    --------------------------------------------------------
  for (std::list<Actor*>::iterator itrexecact = _listOfActor.begin();
       itrexecact != _listOfActor.end(); itrexecact++)
  {
    if ((*itrexecact)->hasMoved())
    {
      DataActor& data = (*itrexecact)->getData();
      _render.addActor((*itrexecact)->getRenderObj(), 
		       data.getXPos(), data.getYPos(), data.getZPos());
    }
  }
  // Puis on regarde les consquences.
  //  1. Le joueur est-il sur un diamant ?
  //  ------------------------------------
  DataActor& plData = _player->getData();
  int x = plData.getXPos();
  int y = plData.getYPos();
  int z = plData.getZPos();
  DataObject* obj = _data.getStaData().getObj(x,y,z);
  if (obj != NULL)
    if (obj->getId() == DIAM)
    {
      Nothing none;
      _nbOfDiamonds--;
      _score += 10;
      rmObject(x,y,z);
      addObject(none, x,y);
    } // Ou a la rigueur du sable :-)
    else if (obj->getId() == SAND)
    {
      Nothing none;
      rmObject(x,y,z);
      addObject(none, x,y);
    }
  //  2. Le joueur a-t'il reu un rocher sur la tronche ?
  //  ---------------------------------------------------
  for (std::list<Actor*>::iterator itrplay = _listOfActor.begin();
       itrplay != _listOfActor.end(); itrplay++)
  {
    DataActor& data = (*itrplay)->getData();
    int ax = data.getXPos();
    int ay = data.getYPos();
    int az = data.getZPos();
    if (((x==ax)&&(y==ay)&&(z==az)&&(data.getFaction()!=plData.getFaction()))&&
	((Monster*)(*itrplay))->isMortal())
    {
      _player->setEnergy(0);
    }
  }
  // Et enfin on rends la scne.
  scr << _render;
  //  while((_nbOfDiamonds!=0)&&(_player->getEnergy()>0));
  return (_nbOfDiamonds==0);
}
