// ===========================================================================
// ==               RoomContainer class : Template Implementation           ==
// ==               Atchoum & X. Pinguin 30 April 2000                      ==
// ===========================================================================
#ifndef _ROOMCONTAINER_H
#define _ROOMCONTAINER_H

#include <vector>

#ifdef _DEBUG
#include <assert.h>
#endif

template<class T>
class RoomContainer
{
public:
  /** */
	typedef std::vector<T> container;
  /** */
  typedef container::iterator  iterator;
  /** */
  typedef container::const_iterator const_iterator;
  /** */
  RoomContainer(int lx, int ly, int lz = 1);
  /** */
  ~RoomContainer();
  /** */
  iterator begin();
  /** */
  const_iterator begin() const;
  /** */
  iterator end();
  /** */
  const_iterator end() const;
  /** */
  T getElt(int x, int y, int z=0);
  /** */
  const T getElt(int x, int y, int z=0) const;
  /** */
  void setElt(const T& elt, int x, int y, int z=0);
  /** */
  int getXLength() const;
  /** */
  int getYLength() const;
  /** */
  int getZLength() const;
private:
  container _data;
  int _lx, _ly, _lz;
};
// ===========================================================================
template<class T>
RoomContainer<T>::RoomContainer(int lx, int ly, int lz) :
  _data(lx*ly*lz),
  _lx(lx), _ly(ly), _lz(lz)
{  
}
// ===========================================================================
template<class T>
RoomContainer<T>::~RoomContainer()
{
}
// ===========================================================================
template<class T> RoomContainer<T>::iterator
RoomContainer<T>::begin()
{
  return _data.begin();
}
// ===========================================================================
template<class T> RoomContainer<T>::const_iterator
RoomContainer<T>::begin() const
{
  return _data.begin();
}
// ===========================================================================
template<class T> RoomContainer<T>::iterator
RoomContainer<T>::end()
{
  return _data.end();
}
// ===========================================================================
template<class T> RoomContainer<T>::const_iterator
RoomContainer<T>::end() const
{
  return _data.end();
}
// ===========================================================================
#ifdef _DEBUG
#define ASSERTACCESS assert(x>=0 ); \
                     assert(x<_lx); \
                     assert(y>=0 ); \
                     assert(y<_ly); \
                     assert(z>=0 ); \
                     assert(z<_lz)
#else
#define ASSERTACCESS
#endif
// ===========================================================================
template<class T> T
RoomContainer<T>::getElt(int x, int y, int z)
{
  ASSERTACCESS;
  return _data[x+y*_lx+z*_lx*_ly];
}
// ===========================================================================
template<class T> const T
RoomContainer<T>::getElt(int x, int y, int z) const
{
  ASSERTACCESS;
  return _data[x+y*_lx+z*_lx*_ly];
}
// ===========================================================================
template<class T> void
RoomContainer<T>::setElt(const T& elt, int x, int y, int z)
{
  ASSERTACCESS;
  _data[x+y*_lx+z*_lx*_ly] = elt;
}
// ===========================================================================
template<class T> inline int
RoomContainer<T>::getXLength() const
{
  return _lx;
}
// ===========================================================================
template<class T> inline int
RoomContainer<T>::getYLength() const
{
  return _ly;
}
// ===========================================================================
template<class T> inline int
RoomContainer<T>::getZLength() const
{
  return _lz;
}
// ===========================================================================
#undef ASSERTACCESS

#endif
